#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <EzGraph.h>
#include "main.h"
#include "layout.h"
#include "puzzle.h"
#include "solver.h"

menu_item_t menu[] = {
    {"õ", -1},
    {"õ", -1},
    {"", -1}, // ߤȥȥ 
    {"STEP", -1},
    {"ο", -1},
    {"ɹ", -1},
    {"¸", -1},
    {"ƥ¸", -1},
    //    {"", -1},
    {"λ", -1},
};

#define MENU_NUM (sizeof(menu)/sizeof(menu_item_t))

int is_running = 0;

char *message = NULL; 

int main(int argc, char *argv[]){
  EzOpenWindow(WIN_WIDTH, WIN_HEIGHT);
  EzSetFontSize(FONT_SIZE);

  init();
  draw_screen();
  EzSetMouseHandler(mouse_h);
 
  EzEventLoop();
  return 0;
}

void init(void){
    clear_area();
    init_solver();
}

void draw_screen(void){
    int bn;
    SetColorWhite();
    for (bn = 0 ; bn < 9 ; bn++){
        draw_block(bn);
    }
    SetColorWhite();
    EzDrawBoxB(AREA_LTX-1, AREA_LTY-1, AREA_RBX+1, AREA_RBY+1);
    draw_focus();
    draw_menu();
    draw_status();
    draw_message();
    EzShowBuffer();
}

void draw_block(int bn){
    int cn;
    int ltx, lty;

    for (cn = 0 ; cn < 9 ; cn++){
        draw_cell(bn, cn);
    }
    ltx = AREA_LTX + n2bx(bn) * BLOCK_SIZE;
    lty = AREA_LTY + n2by(bn) * BLOCK_SIZE;
    SetColorWhite();
    EzDrawBoxB(ltx+1, lty+1, ltx+BLOCK_SIZE-1, lty+BLOCK_SIZE-1);
}


void draw_focus(void){
    int ltx=0, lty=0, rbx=0, rby=0, bn, cx, cy;
    focus_type_t ft;

    ft = get_focus_type();
    if (ft == ft_none) return;

    switch (ft){
    case ft_block:
        bn = get_focus_bn();
        ltx = AREA_LTX + n2bx(bn) * BLOCK_SIZE;
        lty = AREA_LTY + n2by(bn) * BLOCK_SIZE;
        rbx = ltx + BLOCK_SIZE;
        rby = lty + BLOCK_SIZE;
        break;
    case ft_column:
        cx = get_focus_column();
        ltx = AREA_LTX + cx * CELL_SIZE;
        lty = AREA_LTY;
        rbx = ltx + CELL_SIZE;
        rby = lty + BLOCK_SIZE * 3;
        break;
    case ft_row:
        cy = get_focus_row();
        ltx = AREA_LTX;
        lty = AREA_LTY + cy * CELL_SIZE;
        rbx = ltx + BLOCK_SIZE * 3;
        rby = lty + CELL_SIZE;
        break;
    case ft_cell:
        cx = get_focus_column();
        cy = get_focus_row();
        ltx = AREA_LTX + cx * CELL_SIZE;
        lty = AREA_LTY + cy * CELL_SIZE;
        rbx = ltx + CELL_SIZE;
        rby = lty + CELL_SIZE;
        break;
    default:
        break;
    }
    SetColorGreen();
    EzDrawBoxB(ltx, lty, rbx, rby);
    EzDrawBoxB(ltx+1, lty+1, rbx-1, rby-1);
}


void draw_cell(int bn, int cn){
    int ltx, lty;
    int d, ct, dt;
    char buf[2];

    ltx = AREA_LTX + n2cx(bn, cn) * CELL_SIZE;
    lty = AREA_LTY + n2cy(bn, cn) * CELL_SIZE;
    d = get_digit_by_cn(bn, cn);
    ct = get_ctype_by_cn(bn, cn);
    dt = get_dtype_by_cn(bn, cn);
    switch (dt){
    case dt_problem:
        SetColorGray();
        break;
    default:
        SetColorBlack();
        break;
    }
    switch (ct){
    case ct_focused:
        SetColorBlue();
        break;
    case ct_forbidden:
        SetColorRed();
        break;
    case ct_new:
        SetColorYellow();
        break;
    default:
        // ¾ctʤdtǷ᤿Ȥ
        break;
    }
    EzFillBoxB(ltx, lty, ltx + CELL_SIZE, lty + CELL_SIZE);
    SetColorWhite();
    if (d != 0){
        sprintf(buf, "%1d", d);
        EzDrawStringB(ltx+DIGIT_OFFSET_X, lty+DIGIT_OFFSET_Y, buf);
    }
    EzDrawBoxB(ltx, lty, ltx + CELL_SIZE, lty + CELL_SIZE);
}

void draw_menu(void){
    int i, len;

    for (i = 0 ; i < MENU_NUM ; i++){
        SetColorWhite();
        EzDrawBoxB(MENU_LTX, i*MENU_HEIGHT + MENU_LTY,
                   MENU_LTX + MENU_WIDTH,
                   MENU_LTY + i*MENU_HEIGHT + MENU_HEIGHT);
        len = strlen(menu[i].caption);
        if (menu[i].enabled){
            SetColorWhite();
        }
        else {
            SetColorGray();
        }
        EzDrawStringB(MENU_LTX + (MENU_WIDTH - FONT_SIZE/2*len)/2,
                      MENU_LTY + i*MENU_HEIGHT + MENU_STR_OFFSET_Y,
                      menu[i].caption);
    }
}


void draw_status(void){
    int i;
    char buf[256];

    SetColorWhite();
    sprintf(buf, "Ĥ:%2d %5d STEP",
            9*9 - get_digits_total(),
            get_task_count());
    EzDrawStringB(STATUS_LTX, STATUS_LTY + DIGIT_OFFSET_Y, buf);
    for (i = 0 ; i < 9 ; i++){
        buf[i*2] = '1' + i;
        buf[i*2+1] = ' ';
    }
    buf[9*2] = '\0'; 
    EzDrawStringB(STATUS_LTX, STATUS_LTY + DIGIT_OFFSET_Y + CELL_SIZE, buf);
    for (i = 0 ; i < 9 ; i++){
        buf[i*2] = '9' - get_digit_num(i+1);
        buf[i*2+1] = ' ';
    }
    buf[9*2] = '\0'; 
    SetColorWhite();
    EzDrawStringB(STATUS_LTX, STATUS_LTY + DIGIT_OFFSET_Y + CELL_SIZE*2, buf);
}

void draw_message(void){
    if (message != NULL){
        EzDrawStringB(MESSAGE_LTX, MESSAGE_LTY + FONT_SIZE, message);
    }
}

void set_message(char *msg){
    message = msg;
}

void mouse_h(int x, int y, unsigned int n){
    int cx, cy, d, cmd;

    if (x > AREA_LTX && x < AREA_RBX && y > AREA_LTY && y < AREA_RBY){
        x -= AREA_LTX;
        y -= AREA_LTY;
        cx = x / CELL_SIZE;
        cy = y / CELL_SIZE;
        d = get_digit_by_cxy(cx, cy);
        printf("(%d,%d)->c(%d,%d) %d %d\n", x, y, cx, cy, n, d);
        switch (n){
        case 1:
            d = (d + 1) % 10;
            break;
        case 2:
            d = 0;
            break;
        case 3:
            d = (d + 9) % 10;
            break;
        default:
            assert(0);
        }
        set_digit_by_cxy(cx, cy, d);
        if (d != 0){
            set_dtype_by_cxy(cx, cy, dt_problem);
        }
        else {
            set_dtype_by_cxy(cx, cy, dt_none);
        }
    }
    else if (x > MENU_LTX && x < MENU_RBX && y > MENU_LTY && y < MENU_RBY &&
             n == 1)
    {
        y -= MENU_LTY;
        cmd = y / MENU_HEIGHT;
        if (menu[cmd].enabled){
            exec_command(cmd);
        }
    }
    draw_screen();
}

void exec_command(int cmd){
    solver_status_t ss;
    switch (cmd){
    case 0:
        clear_area();
        set_message("õޤ");
        break;
    case 1:
        init_solver();
        set_message("õޤ");
        break;
    case 2:
        toggle_running();
        break;
    case 3:
        while ((ss = solver()) == ss_redo);
        switch (ss){
        case ss_complete:
            set_message("򤱤ޤ");
            clear_ctype();
            clear_focus();
            break;
        case ss_failure:
            set_message("򤱤ޤ");
            clear_ctype();
            clear_focus();
            break;
        default:
            break;
        }
        break;
    case 4:
        while ((ss = solver()) == ss_not_found || ss == ss_redo);
        printf("ss:%d\n", ss);
        switch (ss){
        case ss_complete:
            set_message("򤱤ޤ");
            clear_ctype();
            clear_focus();
            break;
        case ss_failure:
            set_message("򤱤ޤ");
            clear_ctype();
            clear_focus();
            break;
        default:
            break;
        }
        break;
    case 5:
        if (load_area()){
            set_message("ɹ");
        }
        else {
            set_message("ɹ");
        }
        init_solver();
        break;
    case 6:
        if (save_area()){
            set_message("¸");
        }
        else {
            set_message("¸");
        }
        break;
    case 7:
        if (save_text()){
            set_message("ƥ¸");
        }
        else {
            set_message("ƥ¸");
        }
        break;
        //        print_info();
        break;
    case 8:
        printf("bye.\n");
        exit(0);
    default:
        printf("ޥ%d ̤\n", cmd);
        break;
    }
}


void timer_h(void){
    solver_status_t ss;

    while ((ss = solver()) == ss_redo);
    switch (ss){
    case ss_found:
        break;
    case ss_not_found:
        break;
    case ss_complete:
        set_message("򤱤ޤ");
        clear_ctype();
        clear_focus();
        toggle_running();
        break;
    case ss_failure:
        set_message("򤱤ޤ");
        clear_ctype();
        clear_focus();
        toggle_running();
        break;
    default:
        break;
    }
    draw_screen();
}


void toggle_running(void){
    int i;
    if (is_running){
        for (i = 0 ; i < 9 ; i++){
            menu[i].enabled = -1;
        }
        menu[2].caption = "";
        is_running = 0;
        EzSetTimerHandler(NULL, 0);
    }
    else {
        for (i = 0 ; i < 9 ; i++){
            menu[i].enabled = 0;
        }
        menu[2].caption = "";
        menu[2].enabled = -1;
        is_running = -1;
        EzSetTimerHandler(timer_h, TIMER_SPEED);
    }
}
