#include <stdio.h>
#include "puzzle.h"

area_t area, area_org;

count_t count;
focus_t focus;

void clear_area(void){
    int x, y;
    for (y = 0 ; y < 9 ; y++){
        for (x = 0 ; x < 9 ; x++){
            area.cell[x][y].digit = 0;
            area.cell[x][y].ctype = ct_none;
            area.cell[x][y].dtype = dt_none;
        }
    }
    clear_counter();
    clear_focus();
}

void clear_counter(void){
    int i, bn;

    count.digits_total = 0;
    for (i = 0 ; i < 9 ; i++){
        count.digit_num[i] = 0;
    }
    for (bn = 0 ; bn < 9 ; bn++){
        count.digits_in_block[bn] = 0;
        count.digits_in_column[bn] = 0;
        count.digits_in_row[bn] = 0;
    }
}

void clear_focus(void){
    focus.type = ft_none;
}

void set_focus(focus_type_t type, int bn, int column, int row){
    focus.type = type;
    focus.bn = bn;
    focus.column = column;
    focus.row = row;
}

focus_type_t get_focus_type(void){
    return focus.type;
}
int get_focus_bn(void){
    return focus.bn;
}
int get_focus_column(void){
    return focus.column;
}
int get_focus_row(void){
    return focus.row;
}

void clear_ctype(void){
    int x, y;
    for (y = 0 ; y < 9 ; y++){
        for (x = 0 ; x < 9 ; x++){
            area.cell[x][y].ctype = ct_none;
        }
    }
}

int load_area(void){
    FILE *fp;
    int x, y, d;

    clear_area();
    fp = fopen(SAVEFILE, "r");
    if (fp == NULL) return -1;
    for (y = 0 ; y < 9 ; y++){
        for (x = 0 ; x < 9 ; x++){
            if (fscanf(fp, "%d", &d) != 1) return -1;
            if (d != 0){
                set_digit_by_cxy(x, y, d);
                set_dtype_by_cxy(x, y, dt_problem);
            }
        }
    }
    fclose(fp);
    return 0;
}

int save_area(void){
    FILE *fp;
    int x, y;

    fp = fopen(SAVEFILE, "w");
    if (fp == NULL) return -1;
    for (y = 0 ; y < 9 ; y++){
        for (x = 0 ; x < 9 ; x++){
            fprintf(fp, "%d ", get_digit_by_cxy(x, y));
        }
        fprintf(fp, "\n");
    }
    fclose(fp);
    return 0;
}


int save_text(void){
    FILE *fp;
    int x, y, d;

    fp = fopen(TEXTFILE, "w");
    if (fp == NULL) return -1;
    for (y = 0 ; y < 9 ; y++){
        fline(fp);
        for (x = 0 ; x < 9 ; x++){
            d = get_digit_by_cxy(x, y);
            if (d > 0){
                fprintf(fp, "|%d", d);
            }
            else {
                fprintf(fp, "| ");
            }
        }
        fprintf(fp, "|\n");
    }
    fline(fp);
    fclose(fp);
    return 0;
}

void fline(FILE *fp){
    int i;
    for (i = 0 ; i < 9 ; i ++){
        fprintf(fp, "+-");
    }
    fprintf(fp, "+\n");
}



int get_digit_by_cxy(int cx, int cy){
    return area.cell[cx][cy].digit;
}
void set_digit_by_cxy(int cx, int cy, int digit){
    if (area.cell[cx][cy].digit == 0 && digit != 0){
        count.digits_total ++;
        count.digit_num[digit-1] ++;
        count.digits_in_block[cxy2bn(cx, cy)] ++;
        count.digits_in_column[cx] ++;
        count.digits_in_row[cy] ++;
    }
    else if (area.cell[cx][cy].digit != 0 && digit == 0){
        count.digits_total --;
        count.digit_num[area.cell[cx][cy].digit-1] --;
        count.digits_in_block[cxy2bn(cx, cy)] --;
        count.digits_in_column[cx] --;
        count.digits_in_row[cy] --;
    }
    else { // 0ʳΥ¾οѤ
        count.digit_num[area.cell[cx][cy].digit-1] --;
        count.digit_num[digit-1] ++;
    }
    area.cell[cx][cy].digit = digit;
}
cell_type_t get_ctype_by_cxy(int cx, int cy){
    return area.cell[cx][cy].ctype;
}
void set_ctype_by_cxy(int cx, int cy, cell_type_t type){
    area.cell[cx][cy].ctype = type;
}
digit_type_t get_dtype_by_cxy(int cx, int cy){
    return area.cell[cx][cy].dtype;
}
void set_dtype_by_cxy(int cx, int cy, digit_type_t type){
    area.cell[cx][cy].dtype = type;
}

int get_digit_by_cn(int bn, int cn){
    return get_digit_by_cxy(n2cx(bn, cn), n2cy(bn, cn));
}
void set_digit_by_cn(int bn, int cn, int digit){
    set_digit_by_cxy(n2cx(bn, cn), n2cy(bn, cn), digit);
}
cell_type_t get_ctype_by_cn(int bn, int cn){
    return get_ctype_by_cxy(n2cx(bn, cn), n2cy(bn, cn));
}
void set_ctype_by_cn(int bn, int cn, cell_type_t type){
    set_ctype_by_cxy(n2cx(bn, cn), n2cy(bn, cn), type);
}
digit_type_t get_dtype_by_cn(int bn, int cn){
    return get_dtype_by_cxy(n2cx(bn, cn), n2cy(bn, cn));
}
void set_dtype_by_cn(int bn, int cn, digit_type_t type){
    set_dtype_by_cxy(n2cx(bn, cn), n2cy(bn, cn), type);
}

// ֥åbndigitΰ(cn)֤
// Ĥʤ-1֤
int get_cn_in_block(int bn, int digit){
    int cn;
    for (cn = 0 ; cn < 9 ; cn ++){
        if (get_digit_by_cn(bn, cn) == digit){
            return cn;
        }
    } 
    return -1;
}

// ֥åbndigit(cx)֤
// Ĥʤ-1֤
int get_cx_in_block(int bn, int digit){
    int cn;
    cn = get_cn_in_block(bn, digit);
    if (cn == -1){
        return -1;
    }
    else {
        return n2cx(bn, cn);
    }
}
// ֥åbndigitι(cy)֤
// Ĥʤ-1֤
int get_cy_in_block(int bn, int digit){
    int cn;
    cn = get_cn_in_block(bn, digit);
    if (cn == -1){
        return -1;
    }
    else {
        return n2cy(bn, cn);
    }
}

// rowdigit(cx)֤
// Ĥʤ-1֤
int get_cx_in_row(int row, int digit){
    int cx;
    for (cx = 0 ; cx < 9 ; cx ++){
        if (get_digit_by_cxy(cx, row) == digit){
            return cx;
        }
    } 
    return -1;
}
// columndigitι(cy)֤
// Ĥʤ-1֤
int get_cy_in_column(int column, int digit){
    int cy;
    for (cy = 0 ; cy < 9 ; cy ++){
        if (get_digit_by_cxy(column, cy) == digit){
            return cy;
        }
    } 
    return -1;
}

// ֥åbnͣζޥ(äƤ餺forbiddenǤʤ)֤
// 䤬ʤʣ-1֤
int get_free_cell_in_block(int bn){
    int cn, free_cn = -1;
    for (cn = 0 ; cn < 9 ; cn++){
        if (get_digit_by_cn(bn, cn) == 0 &&
            get_ctype_by_cn(bn, cn) != ct_forbidden)
        {
            //            printf("getfree%d\n", cn);
            if (free_cn == -1){
                free_cn = cn;
            }
            else {
                return -1;
            }
        }
    }
    return free_cn;
}
// columnͣζޥ(äƤ餺forbiddenǤʤ)֤
// 䤬ʤʣ-1֤
int get_free_cell_in_column(int column){
    int cy, free_cy = -1;
    for (cy = 0 ; cy < 9 ; cy++){
        if (get_digit_by_cxy(column, cy) == 0 &&
            get_ctype_by_cxy(column, cy) != ct_forbidden)
        {
            if (free_cy == -1){
                free_cy = cy;
            }
            else {
                return -1;
            }
        }
    }
    return free_cy;
}
// rowͣζޥ(äƤ餺forbiddenǤʤ)֤
// 䤬ʤʣ-1֤
int get_free_cell_in_row(int row){
    int cx, free_cx = -1;
    for (cx = 0 ; cx < 9 ; cx++){
        if (get_digit_by_cxy(cx, row) == 0 &&
            get_ctype_by_cxy(cx, row) != ct_forbidden)
        {
            if (free_cx == -1){
                free_cx = cx;
            }
            else {
                return -1;
            }
        }
    }
    return free_cx;
}

// ֥åbnΥΤ8ĤޤǤϳꤷƤ뤳
int get_last_digit_in_block(int bn){
    int cn, sum = 0;
    for (cn = 0 ; cn < 9 ; cn++){
        sum += get_digit_by_cn(bn, cn);
    }
    return 45-sum;
}

// columnΥΤ8ĤޤǤϳꤷƤ뤳
int get_last_digit_in_column(int column){
    int cy, sum = 0;
    for (cy = 0 ; cy < 9 ; cy++){
        sum += get_digit_by_cxy(column, cy);
    }
    return 45-sum;
}
// rowΥΤ8ĤޤǤϳꤷƤ뤳
int get_last_digit_in_row(int row){
    int cx, sum = 0;
    for (cx = 0 ; cx < 9 ; cx++){
        sum += get_digit_by_cxy(cx, row);
    }
    return 45-sum;
}
// ֥åbnγζ֤ӥå֤
// ֤ͤ23ǡӥå13ޥȤޤäƤ
int get_full_column_in_block(int bn){
    int bits = 0;
    int cx, cy, sum;

    for (cx = cxmin(bn) ; cx <= cxmax(bn) ; cx++){
        sum = 0;
        for (cy = cymin(bn) ; cy <= cymax(bn) ; cy++){
           if (get_digit_by_cxy(cx, cy) > 0){
               sum ++;
           }
        }
        if (sum == 3){
            set_bits(bits, cx-cxmin(bn));
        }
    }
    return bits;
}
// ֥åbnζ֤ӥå֤
// ֤ͤ23ǡӥå1ιԤ3ޥȤޤäƤ
int get_full_row_in_block(int bn){
    int bits = 0;
    int cx, cy, sum;

    for (cy = cymin(bn) ; cy <= cymax(bn) ; cy++){
        sum = 0;
        for (cx = cxmin(bn) ; cx <= cxmax(bn) ; cx++){
           if (get_digit_by_cxy(cx, cy) > 0){
               sum ++;
           }
        }
        if (sum == 3){
            set_bits(bits, cy-cymin(bn));
        }
    }
    return bits;
}





int get_digits_total(void){
    return count.digits_total;
}

int get_digit_num(int digit){
    return count.digit_num[digit-1];
}
int get_digits_in_block(int bn){
    return count.digits_in_block[bn];
}
int get_digits_in_column(int column){
    return count.digits_in_column[column];
}
int get_digits_in_row(int row){
    return count.digits_in_row[row];
}

void forbid_by_cxy(int cx, int cy){
    if (get_dtype_by_cxy(cx, cy) == dt_none){
        set_ctype_by_cxy(cx, cy, ct_forbidden);
    }
}

// ֥åbnιcyforbiddenˤ
void forbid_row_in_block(int bn, int cy){
    int cx;
    //    printf("forbid %d %d\n", bn, cy);
    for (cx = cxmin(bn) ; cx <= cxmax(bn) ; cx++){
        forbid_by_cxy(cx, cy);
    }
}
// ֥åbncxforbiddenˤ
void forbid_column_in_block(int bn, int cx){
    int cy;
    //    printf("forbid %d %d\n", bn, cx);
    for (cy = cymin(bn) ; cy <= cymax(bn) ; cy++){
        forbid_by_cxy(cx, cy);
    }
}


void print_info(void){
    int bn, column, row;
    printf(":%d\n", count.digits_total);
    printf("ƥ֥åοĿ:");
    for (bn = 0 ; bn < 9 ; bn++){
        printf("%d ", count.digits_in_block[bn]);
    }
    printf("οĿ:");
    for (column = 0 ; column < 9 ; column++){
        printf("%d ", count.digits_in_column[column]);
    }
    printf("ƹοĿ:");
    for (row = 0 ; row < 9 ; row++){
        printf("%d ", count.digits_in_row[row]);
    }
    printf("\n");
}

int get_free_digit_in_bits(unsigned int bits){
    int i, digit = 0;
    for (i = 0 ; i < 9 ; i++){
        if ((bits & (1 << i)) == 0){
            if (digit == 0){
                digit = i+1;
            }
            else {
                return 0;
            }
        }
    }
    printf("free digit:%d\n", digit);
    return digit;
}
